<?php

namespace Teknisa\CADCTB\Controllers;

use Teknisa\CADCTB\Services\Ctb94000AtualizacaoCustoProducaoService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request\DataSet as Data;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\CADCTB\Controllers\TeknisaCrud;
use Teknisa\CADCTB\Controllers\EntityManager;
use Teknisa\Libs\Util\Environment;

class Ctb94000AtualizacaoCustoProducaoController{
    
    protected $ctb94000AtualizacaoCustoProducaoService;
    protected $entityManager;
    protected $connection;
    
    public function __construct(\Doctrine\ORM\EntityManager $entityManager, Environment $environment, Ctb94000AtualizacaoCustoProducaoService $ctb94000AtualizacaoCustoProducaoService){
        $this->ctb94000AtualizacaoCustoProducaoService = $ctb94000AtualizacaoCustoProducaoService;
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
    }
    
    public function atualizaCustoProd(Row $row, Response $response) {
        $row = $row->getRow();
        $this->entityManager->beginTransaction();
        try{
            $this->ctb94000AtualizacaoCustoProducaoService->verificaHistorico();
            $this->ctb94000AtualizacaoCustoProducaoService->verificaPeriodo(array(
                    "CDFILIAL" => $row['CDFILIAL'],
                    "DTLANC" => '01/'.$row['DTLANC']
                ));
            //$row['CDPRODFIN'] = str_pad ($row['CDPRODFIN'], 13, 'Z');
            $row['CDPRODFIN'] = $row['CDPRODFIN'].'ZZZZZZZZZZZZZ'; 
                
            $this->ctb94000AtualizacaoCustoProducaoService->execAtualizaCustoAbsorcao($row);
        }
        catch (\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function cancelaCustoProd(Row $row, Response $response) {
        $row = $row->getRow();
        $this->entityManager->beginTransaction();
        try{
            /*$this->ctb94000AtualizacaoCustoProducaoService->verificaHistorico();
            $this->ctb94000AtualizacaoCustoProducaoService->verificaPeriodo(array(
                    "CDFILIAL" => $row['CDFILIAL'],
                    "DTLANC" => '01/'.$row['DTLANC']
                ));*/
            $this->ctb94000AtualizacaoCustoProducaoService->execCancelaCustoAbsorcao($row);
        }
        catch (\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
}