<?php
namespace Teknisa\CADCTB\Services;

use Teknisa\Libs\Util\Environment;
use Doctrine\ORM\EntityManager;
use Teknisa\CADCTB\Factories\Ger03500ParametrosCentroDeCustoFactory;

/**
 * Class Ger03500ParametrosCentroDeCustoService
 * @package Teknisa\CADCTB\Services
 */
class Ger03500ParametrosCentroDeCustoService {

    protected $entityManager;
    protected $environment;
    protected $factory;

    /**
     * @param EntityManager                          $entityManager
     * @param Environment                            $environment
     * @param Ger03500ParametrosCentroDeCustoFactory $factory
     */
    public function __construct(EntityManager $entityManager, Environment $environment, Ger03500ParametrosCentroDeCustoFactory $factory) {
        $this->entityManager = $entityManager;
        $this->environment   = $environment;
        $this->factory       = $factory;
    }

    /**
     * Gera Rateio de Centro de Custo
     */
    public function geraRateioCentroDeCusto($name, $tipo, $rateios) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            // Busca Rateios Anteriores
            if($name == 'RateioTipoContaPagarGrid')
                $rateios_old = $this->factory->buscaRattpctp($tipo['CDTPCTPAGAR']);
            if($name == 'RateioTipoAjusteContaPagarGrid')
                $rateios_old = $this->factory->buscaRattpajp($tipo['CDAJUCTPAG']);
            if($name == 'RateioTipoContaReceberGrid')
                $rateios_old = $this->factory->buscaRattpctr($tipo['CDTPCTRECEB']);
            if($name == 'RateioTipoAjusteContaReceberGrid')
                $rateios_old = $this->factory->buscaRattpajr($tipo['CDAJUCTREC']);
            if($name == 'RateioTipoDebitoBancarioGrid')
                $rateios_old = $this->factory->buscaRattpdebi($tipo['CDTPDEBIBANC']);
            if($name == 'RateioTipoCreditoBancarioGrid')
                $rateios_old = $this->factory->buscaRattpcred($tipo['CDTPCREDBANC']);
            
            // Exclui Rateios Anteriores
            if($rateios_old)
                foreach($rateios_old as $rateio_old)
                    $this->entityManager->remove($rateio_old);
            $this->entityManager->flush();
            
            // Insere Novos Rateios
            foreach($rateios as $rateio){
                if($name == 'RateioTipoContaPagarGrid')
                    $rateio = $this->factory->insereRattpctp($tipo['NRORG'], $tipo['CDTPCTPAGAR'], $rateio['CDCENTCUST'], $this->formatFloat($rateio['VRPERATEIO']));
                if($name == 'RateioTipoAjusteContaPagarGrid')
                    $rateio = $this->factory->insereRattpajp($tipo['NRORG'], $tipo['CDAJUCTPAG'], $rateio['CDCENTCUST'], $this->formatFloat($rateio['VRPERATEIO']));
                if($name == 'RateioTipoContaReceberGrid')
                    $rateio = $this->factory->insereRattpctr($tipo['NRORG'], $tipo['CDTPCTRECEB'], $rateio['CDCENTCUST'], $this->formatFloat($rateio['VRPERATEIO']));
                if($name == 'RateioTipoAjusteContaReceberGrid')
                    $rateio = $this->factory->insereRattpajr($tipo['NRORG'], $tipo['CDAJUCTREC'], $rateio['CDCENTCUST'], $this->formatFloat($rateio['VRPERATEIO']));
                if($name == 'RateioTipoDebitoBancarioGrid')
                    $rateio = $this->factory->insereRattpdebi($tipo['NRORG'], $tipo['CDTPDEBIBANC'], $rateio['CDCENTCUST'], $this->formatFloat($rateio['VRPERATEIO']));
                if($name == 'RateioTipoCreditoBancarioGrid')
                    $rateio = $this->factory->insereRattpcred($tipo['NRORG'], $tipo['CDTPCREDBANC'], $rateio['CDCENTCUST'], $this->formatFloat($rateio['VRPERATEIO']));
                
                $this->entityManager->persist($rateio);
            }
            
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Exclui Rateio de Centro de Custo
     */
    public function excluiRateioCentroDeCusto($name, $tipo) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            // Busca Rateios Anteriores
            if($name == 'RateioTipoContaPagarGrid')
                $rateios_old = $this->factory->buscaRattpctp($tipo['CDTPCTPAGAR']);
            if($name == 'RateioTipoAjusteContaPagarGrid')
                $rateios_old = $this->factory->buscaRattpajp($tipo['CDAJUCTPAG']);
            if($name == 'RateioTipoContaReceberGrid')
                $rateios_old = $this->factory->buscaRattpctr($tipo['CDTPCTRECEB']);
            if($name == 'RateioTipoAjusteContaReceberGrid')
                $rateios_old = $this->factory->buscaRattpajr($tipo['CDAJUCTREC']);
            if($name == 'RateioTipoDebitoBancarioGrid')
                $rateios_old = $this->factory->buscaRattpdebi($tipo['CDTPDEBIBANC']);
            if($name == 'RateioTipoCreditoBancarioGrid')
                $rateios_old = $this->factory->buscaRattpcred($tipo['CDTPCREDBANC']);
            
            // Exclui Rateios Anteriores
            if($rateios_old)
                foreach($rateios_old as $rateio_old)
                    $this->entityManager->remove($rateio_old);
            $this->entityManager->flush();
            
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Cadastramento Automático Centro de Custo
     */
    public function geraCadastramentoAutomaticoCentroDeCusto($name, $data) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            // Busca Produtos
            if($name == 'ProdutoGrid')
                $produtos = $this->factory->buscaProdutos($data['CDARVPRODINI'], $data['CDARVPRODFIN']);
            if($name == 'ProdutoPorUnidadeGrid')
                $produtos = $this->factory->buscaProdfilis($data['CDARVPRODINI'], $data['CDARVPRODFIN'], $data['CDFILIAL']);

            if($produtos){
                foreach($produtos as $produto){
                    if($name == 'ProdutoGrid'){
                        // Insere Centros de Custo Produto
                        $produto = $this->factory->buscaProduto($produto['CDPRODUTO']);
                        $data['CDCCUSTENTR'] = isset($data['CDCCUSTENTR'])?$data['CDCCUSTENTR']:null;
                        $data['CDCCUSTSAID'] = isset($data['CDCCUSTSAID'])?$data['CDCCUSTSAID']:null;
                        $produto = $this->factory->atualizaProduto($produto, $data['CDCCUSTENTR'], $data['CDCCUSTSAID']);
                    }
                    if($name == 'ProdutoPorUnidadeGrid'){
                        // Insere Centros de Custo Produto por Filial
                        $produto = $this->factory->buscaProdfili($produto['CDPRODUTO'], $produto['CDFILIAL']);
                        $data['CDCCUSTENTR'] = isset($data['CDCCUSTENTR'])?$data['CDCCUSTENTR']:null;
                        $data['CDCCUSTSAID'] = isset($data['CDCCUSTSAID'])?$data['CDCCUSTSAID']:null;
                        $produto = $this->factory->atualizaProdfili($produto, $data['CDCCUSTENTR'], $data['CDCCUSTSAID']);
                    }
                    $this->entityManager->persist($produto);
                }
            }
            $this->entityManager->flush();
            
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    /**
     * Formata Número
     */
    public function formatFloat($value) {
        return floatval (str_replace(',', '.', $value));
    }

}