<?php

namespace Teknisa\CADCTB\Services;

use Zeedhi\Framework\DTO\Response;
use Teknisa\Libs\Util\DateUtil;
use Zeedhi\Framework\DataSource\DataSet;
use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Sap03200PeriodoFechamentoContabilGerencialService{

    protected $environment;
    protected $nrorg;
    
    
    public function __construct(EntityManager $entityManager, Environment $environment) {
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->nrorg = $environment->getNrOrgTrab();
        $this->connection = $entityManager->getConnection();
    }

    
    public function contabilizaSelecao($row, $filiaisArray){
        try{
            foreach ($filiaisArray as $filial){
                $row['CDFILIAL']= $filial;
                $this->execContabilizaSelecao($row);
            }
        }
        catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getFiliaisFechamento($row){
        try{
            // Query a ser executada.
            $sql = "SELECT DISTINCT C.CDFILIAL
                        FROM
                        (SELECT L.*
                          FROM LANCCCUSTO L JOIN NOTAFISCAL N ON L.NRLANCCONT = N.NRLANCTBDOC
                         WHERE N.DTENTRSAID BETWEEN :DTINI AND :DTFIN
                           AND EXISTS (SELECT 1
                                         FROM PERCTBGERFIL P
                                        WHERE P.CDFILIAL = DECODE(N.IDENTRSAIDOP, 'E', N.CDFILIRECE, N.CDFILIEMIT)
                                          AND L.CDFILIAL = P.CDFILIAL
                                          AND TO_NUMBER(TO_CHAR(N.DTENTRSAID, 'DD')) BETWEEN P.NRDIAINIPER AND P.NRDIAFIMPER)) C,  CCTBLGERFIL CC, CONTCTBL CONT
                        WHERE CC.CDFILIAL   = C.CDFILIAL
                          AND CC.CDCCTBREFE = C.CDCONTCTBL
                          AND C.CDCONTCTBL  = CONT.CDCONTCTBL
                          AND CONT.IDCONTCCUS <> 'O'";
    
            // Prepara o statement da query para execução.
            $statement = $this->connection->prepare($sql);
            
            // Executa o statement com os paràmetros passados.
            $statement->execute(array(
                "DTINI" => $row['DTINICIAL'],
                "DTFIN" => $row['DTFINAL']
            ));

            // Executa um fetch em todos os dados, retornando-os.
            $filiaisArray = $statement->fetchAll();
            if(empty($filiaisArray)){
                throw new \Exception("Não foram encontradas filiais para executar a tarefa.",1);
            }else{
                return $filiaisArray;
            }
            
        }
        catch (\Exception $e) {
            throw $e;
        }
    }
    
        
    public function execContabilizaSelecao($row){
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'ATUALIZA_DATACOMP_GERENCIAL');
        $procedure->addParam(new Param('P_CDFILIAL',  Param::PARAM_INPUT, $row['CDFILIAL'],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTINICIAL', Param::PARAM_INPUT, $row['DTINICIAL'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTFINAL'  , Param::PARAM_INPUT, $row['DTFINAL'],   Param::PARAM_TYPE_STR));
        $output = $procedure->execute();
    }
    
    
}